__all__ = (
    "Ability",
    "CardType",
    "Color",
    "Constants",
    "Expansion",
    "Rarity",
)

import enum
import os


class Constants:
    HTML_FOLDER = os.path.join(os.path.dirname(__file__), "html")
    EXPANSIONS_FOLDER = os.path.join(HTML_FOLDER, "expansions")
    JSON_FOLDER = os.path.join(os.path.dirname(__file__), "json")


class Color(enum.StrEnum):
    GRASS = "Grass"
    FIRE = "Fire"
    WATER = "Water"
    LIGHTNING = "Lightning"
    PSYCHIC = "Psychic"
    FIGHTING = "Fighting"
    DARKNESS = "Darkness"
    METAL = "Metal"
    DRAGON = "Dragon"
    COLORLESS = "Colorless"


class CardType(enum.StrEnum):
    POKEMON = "Pokémon"
    ITEM = "Item"
    SUPPORTER = "Supporter"


class Rarity(enum.StrEnum):
    COMMON = "◊"
    UNCOMMON = "◊◊"
    RARE = "◊◊◊"
    EX = "◊◊◊◊"
    FULL_ART = "☆"
    FULL_ART_RARE = "☆☆"
    IMMERSIVE = "☆☆☆"
    CROWN = "🜲"


class Expansion(enum.StrEnum):
    GENETIC_APEX = "A1"
    MYTHICAL_ISLAND = "A1a"
    PROMO_A = "P-A"


class Ability(enum.StrEnum):
    WITH_ABILITY = "With ability"
    WITHOUT_ABILITY = "Without ability"



__all__ = ("app",)

import dash
import dash_mantine_components as dmc

dash._dash_renderer._set_react_version("18.2.0")  # noqa

app = dash.Dash(
    title="PTCGP Explorer",
    external_stylesheets=dmc.styles.ALL,
    use_pages=True,
)

app.layout = dash.page_container
app.server.static_folder = "assets"


if __name__ == "__main__":
    app.run(debug=True)



__all__ = ("layout",)

import dash
import dash_mantine_components as dmc

from ... import constants, parser
from ..components import Select

dash.register_page(__name__, path="/")

ALL_CARDS = sorted(parser.parse_all_cards(), key=lambda c: c.index_str)

carousel = dmc.Carousel(
    children=[],
    id="carousel-cards",
    withIndicators=True,
    slideSize={"base": "100%", "sm": "50%", "md": "33.333333%"},
    slideGap={"base": "sm", "sm": "md"},
    loop=True,
    align="start",
    height=500,
    p=50,
)

card_filters = dmc.Group(
    [
        Select("color", constants.Color),
        Select("rarity", constants.Rarity),
        Select("card type", constants.CardType),
        Select("weakness", constants.Color),
        dmc.Select(
            data=list(constants.Ability),
            id="select-ability",
            placeholder="Select ability...",
            clearable=True,
        ),
        Select("expansion", constants.Expansion),
        dmc.Button(
            children="Clear all",
            id="clear-filters-btn",
            color="gray",
            variant="light",
        ),
    ]
)

layout = dmc.MantineProvider(
    [
        dash.html.H1(
            children="Pokémon TCG Pocket Explorer", style={"textAlign": "center"}
        ),
        dmc.Center(card_filters),
        carousel,
        dmc.Center(dash.html.H4(id="card-count", children="")),
    ]
)


@dash.callback(
    dash.Output("carousel-cards", "children"),
    dash.Output("card-count", "children"),
    dash.Input("select-color", "value"),
    dash.Input("select-rarity", "value"),
    dash.Input("select-card-type", "value"),
    dash.Input("select-weakness", "value"),
    dash.Input("select-ability", "value"),
    dash.Input("select-expansion", "value"),
)
def filter_cards(
    color_values: list[str],
    rarity_values: list[str],
    card_types: list[str],
    weaknesses: list[str],
    ability: str,
    expansions: list[str],
):
    cards = ALL_CARDS

    if color_values:
        cards = [c for c in cards if c.color in color_values]
    if rarity_values:
        cards = [c for c in cards if c.rarity in rarity_values]
    if card_types:
        cards = [c for c in cards if c.card_type in card_types]
    if weaknesses:
        cards = [c for c in cards if c.weakness in weaknesses]
    if ability is not None:
        if ability == constants.Ability.WITH_ABILITY:
            cards = [c for c in cards if c.ability is not None]
        else:
            cards = [c for c in cards if c.ability is None]
    if expansions:
        cards = [c for c in cards if c.expansion in expansions]

    return [dmc.CarouselSlide(c.as_ui()) for c in cards], f"{len(cards)} matching cards"


dash.clientside_callback(
    """function (n) { return [[], [], [], [], null, []]; }""",
    dash.Output("select-color", "value"),
    dash.Output("select-rarity", "value"),
    dash.Output("select-card-type", "value"),
    dash.Output("select-weakness", "value"),
    dash.Output("select-ability", "value"),
    dash.Output("select-expansion", "value"),
    dash.Input("clear-filters-btn", "n_clicks"),
    prevent_initial_call=True,
)



__all__ = (
    "Ability",
    "Attack",
    "Card",
)

import os

import bs4
import dash
import dash_mantine_components as dmc
import pydantic

from . import constants
from . import utils


class Attack(pydantic.BaseModel):
    name: str
    cost: str
    effect: str | None = None
    damage: int | None = None
    damage_min: int | None = None
    damage_mult: int | None = None

    @classmethod
    def from_soup(cls, soup: bs4.Tag) -> "Attack":
        atk_parts = soup.find("p", class_="card-text-attack-info").text.split()

        args = {
            "cost": soup.find("p", class_="card-text-attack-info")
            .find("span", class_="ptcg-symbol")
            .text,
            "name": atk_parts[1],
            "effect": soup.find("p", class_="card-text-attack-effect").text.strip(),
        }

        try:
            dmg = atk_parts[-1]
            if "+" in dmg:
                args["damage_min"] = int(dmg.strip("+"))
            elif "x" in dmg:
                args["damage_mult"] = int(dmg.strip("x"))
            else:
                args["damage"] = int(dmg)
        except:
            pass

        return cls.model_validate(args)


class Ability(pydantic.BaseModel):
    name: str
    effect: str

    @classmethod
    def from_soup(cls, soup: bs4.Tag) -> "Ability":
        args = {
            "name": soup.find("p", class_="card-text-ability-info")
            .text.split("Ability: ")[-1]
            .strip(),
            "effect": soup.find("p", class_="card-text-ability-effect").text.strip(),
        }

        return cls.model_validate(args)


class Card(pydantic.BaseModel):
    id: str
    expansion: str
    image: str
    name: str
    card_type: constants.CardType
    artist: str
    attacks: list[Attack] = pydantic.Field(default_factory=list)

    rarity: constants.Rarity | None = None
    color: constants.Color | None = None
    life: int | None = None
    ability: Ability | None = None
    weakness: constants.Color | None = None
    retreat_cost: int | None = None

    @property
    def index_str(self) -> str:
        return f"{self.expansion}_{int(self.id):03}"

    @classmethod
    def get_color(cls, title: list[str]) -> constants.Color | None:
        for member in title:
            if member in constants.Color:
                return constants.Color(member)

    @classmethod
    def get_type(cls, soup: bs4.Tag) -> constants.CardType:
        for card_type in constants.CardType:
            if card_type in soup.text:
                return constants.CardType(card_type)

    @classmethod
    def get_rarity(cls, soup: bs4.Tag) -> constants.Rarity:

        if "Crown Rare" in soup.text:
            return constants.Rarity.CROWN
        else:
            for rarity in [
                constants.Rarity.EX,
                constants.Rarity.RARE,
                constants.Rarity.UNCOMMON,
                constants.Rarity.COMMON,
                constants.Rarity.IMMERSIVE,
                constants.Rarity.FULL_ART_RARE,
                constants.Rarity.FULL_ART,
            ]:
                if rarity in soup.text:
                    return rarity

    @classmethod
    def from_html(cls, html: str) -> "Card":
        soup = bs4.BeautifulSoup(html, features="html.parser")
        profile = soup.find("div", class_="card-profile")
        details = profile.find("div", class_="card-details")
        title = " ".join(
            details.find("p", class_="card-text-title").text.split()
        ).split(" - ")
        print_info = soup.find("div", class_="card-prints-current")
        card_id = (
            print_info.find("div", class_="prints-current-details")
            .find_all("span")[1]
            .text.split()[0]
            .strip("#")
        )

        args = {
            "id": card_id,
            "expansion": print_info.img.get("alt"),
            "image": profile.find("div", class_="card-image").img.get("src"),
            "name": details.find("span", class_="card-text-name").text,
            "card_type": cls.get_type(details.find("p", class_="card-text-type")),
            "rarity": cls.get_rarity(print_info),
            "color": cls.get_color(title),
            "artist": details.find("div", class_="card-text-artist").a.text.strip(),
            "attacks": [],
        }

        if weakness_retreat := details.find("p", class_="card-text-wrr"):
            _, weakness, _, retreat_cost = weakness_retreat.text.split()
            if weakness != "none":
                args["weakness"] = weakness
            args["retreat_cost"] = retreat_cost

        for attack in details.find_all("div", class_="card-text-attack"):
            args["attacks"].append(Attack.from_soup(attack))

        if ability := details.find("div", class_="card-text-ability"):
            args["ability"] = Ability.from_soup(ability)

        try:
            args["life"] = int(title[-1].strip(" HP"))
        except:
            pass

        return cls.model_validate(args)

    def to_file(self):
        utils.mkdir(constants.Constants.JSON_FOLDER)
        filename = f"{self.index_str}.json"
        filepath = os.path.join(constants.Constants.JSON_FOLDER, filename)
        with open(filepath, "w", encoding="utf-8") as file:
            n = file.write(self.model_dump_json(indent=2))
            print(f"wrote {n} to {filepath}")

    def as_ui(self):
        img = dmc.Image(src=self.image, alt=self.name, fit="scale-down", h=500)
        return dash.html.A(
            href=f"/card/{self.index_str}",
            children=[img],
        )



__all__ = (
    "cards_urls",
    "download_all_cards",
    "download_card_html",
    "download_expansion_html",
    "download_expansions_html",
    "expansion_urls",
)

import enum
import http
import os
import typing

import requests
from bs4 import BeautifulSoup

from . import constants
from . import utils


class Urls(enum.StrEnum):
    BASE = "https://pocket.limitlesstcg.com"
    EXPANSIONS = f"{BASE}/cards/"


def download_expansions_html() -> str:
    res = requests.get(Urls.EXPANSIONS)

    if res.status_code != http.HTTPStatus.OK:
        raise Exception(res.status_code)

    filename = os.path.join(constants.Constants.HTML_FOLDER, "expansions.html")
    utils.mkdir(constants.Constants.HTML_FOLDER)
    with open(filename, "w", encoding="utf-8") as file:
        n = file.write(res.text)
        print(f"wrote {n} to {filename}")

    return res.text


def expansion_urls(text: str | None = None) -> typing.Iterator[str]:
    html_text = (
        text or utils.load_html_file("expansions.html") or download_expansions_html()
    )
    soup = BeautifulSoup(html_text, features="html.parser")
    data_table = soup.find("table", class_="data-table")
    routes = set(link.get("href") for link in data_table.find_all("a"))
    for route in sorted(routes):
        yield Urls.BASE + route


def download_expansion_html(exp_url: str) -> str:
    exp_name = exp_url.split("/")[-1]
    res = requests.get(exp_url)

    if res.status_code != http.HTTPStatus.OK:
        raise Exception(res.status_code)

    filename = os.path.join(constants.Constants.EXPANSIONS_FOLDER, f"{exp_name}.html")
    utils.mkdir(constants.Constants.EXPANSIONS_FOLDER)
    with open(filename, "w", encoding="utf-8") as file:
        n = file.write(res.text)
        print(f"wrote {n} to {filename}")

    return res.text


def cards_urls(exp_url: str) -> typing.Iterator[str]:
    exp_name = exp_url.split("/")[-1]
    html_text = utils.load_html_file(
        f"expansions/{exp_name}.html"
    ) or download_expansion_html(exp_url)

    soup = BeautifulSoup(html_text, features="html.parser")
    cards_table = soup.find("div", class_="card-search-grid")
    routes = set(link.get("href") for link in cards_table.find_all("a"))
    for route in sorted(routes):
        yield Urls.BASE + route


def download_card_html(card_url: str) -> str:
    res = requests.get(card_url)

    if res.status_code != http.HTTPStatus.OK:
        raise Exception(res.status_code)

    exp_name = card_url.split("/")[-2]
    card_id = card_url.split("/")[-1]
    exp_folder = os.path.join(constants.Constants.EXPANSIONS_FOLDER, exp_name)
    utils.mkdir(exp_folder)
    filename = os.path.join(exp_folder, f"{card_id}.html")
    with open(filename, "w", encoding="utf-8") as file:
        n = file.write(res.text)
        print(f"wrote {n} to {filename}")

    return res.text


def download_all_cards():
    for exp_url in expansion_urls():
        for card_url in cards_urls(exp_url):
            download_card_html(card_url)


if __name__ == "__main__":
    download_all_cards()



__all__ = (
    "dash_options",
    "kebab_case",
    "load_file",
    "load_html_file",
    "mkdir",
)

import enum
import os

from .constants import Constants


def load_html_file(filename: str) -> str | None:
    filepath = os.path.join(Constants.HTML_FOLDER, filename)
    return load_file(filepath)


def load_file(filepath: str) -> str | None:
    try:
        with open(filepath, encoding="utf-8") as file:
            return file.read()
    except:
        return None


def mkdir(path: str):
    try:
        os.mkdir(path)
    except:
        pass


def dash_options(enumeration: type[enum.StrEnum]):
    return [
        {"label": str(member.name), "value": str(member.value)}
        for member in list(enumeration)
    ]


def kebab_case(input_str: str) -> str:
    return input_str.lower().replace("_", "-").replace(" ", "-")



__all__ = ("layout",)

import dash
import dash_mantine_components as dmc

from ... import parser

dash.register_page(__name__, path_template="/card/<card_str>")


def layout(card_str: str, **kwargs):
    exp_name, card_id = card_str.split("_", maxsplit=1)
    card = parser.parse_card_by_id(exp_name, card_id)
    return dmc.Image(src=card.image)



__all__ = (
    "parse_card_by_id",
    "parse_card",
    "parse_all_cards",
)

import os
import typing

from . import constants
from . import objects
from . import utils


def parse_card_by_id(exp_name: str, card_id: str) -> objects.Card:
    exp_folder = os.path.join(constants.Constants.EXPANSIONS_FOLDER, exp_name)
    filepath = os.path.join(exp_folder, f"{card_id}.html")
    return parse_card(filepath)


def parse_card(filepath: str) -> objects.Card:
    html = utils.load_file(filepath)
    return objects.Card.from_html(html)


def parse_all_cards() -> typing.Iterator[objects.Card]:
    for dir_path, _dir_names, files in os.walk(constants.Constants.EXPANSIONS_FOLDER):
        if dir_path == constants.Constants.EXPANSIONS_FOLDER:
            continue

        for file in files:
            file_path = os.path.join(dir_path, file)
            try:
                yield parse_card(str(file_path))
            except Exception:
                print(f"failed to parse file {file_path}")
                raise


if __name__ == "__main__":
    for card in parse_all_cards():
        card.to_file()



from setuptools import setup, find_packages

setup(
    name="ptcgp",
    version="0.1.0",
    url="https://github.com/fredericojordan/ptcgp.git",
    author="Frederico Jordan",
    author_email="fredericojordan@gmail.com",
    description="Pokémon TCG Pocket Explorer App",
    packages=find_packages(),
    install_requires=[
        "beautifulsoup4",
        "dash",
        "dash_mantine_components",
        "pydantic",
        "requests",
    ],
)



__all__ = ("Select",)

import enum

import dash_mantine_components as dmc

from ... import utils


class Select(dmc.MultiSelect):
    def __init__(self, name: str, options: type[enum.StrEnum], *args, **kwargs):
        kwargs = {
            "id": f"select-{utils.kebab_case(name)}",
            "data": list(options),
            "placeholder": f"Select {name}...",
            "searchable": True,
            "clearable": True,
            **kwargs,
        }
        super().__init__(*args, **kwargs)
